@@ -75,6 +75,9 @@ static SDL_HIDAPI_DeviceDriver *SDL_HIDAPI_drivers[] = {
     &SDL_HIDAPI_DriverJoyCons,
     &SDL_HIDAPI_DriverSwitch,
 #endif
+#ifdef SDL_JOYSTICK_HIDAPI_SWITCH2
+    &SDL_HIDAPI_DriverSwitch2,
+#endif
 #ifdef SDL_JOYSTICK_HIDAPI_WII
     &SDL_HIDAPI_DriverWii,
 #endif
@@ -82,9 +85,27 @@ static SDL_HIDAPI_DeviceDriver *SDL_HIDAPI_drivers[] = {
     &SDL_HIDAPI_DriverXbox360,
     &SDL_HIDAPI_DriverXbox360W,
 #endif
+#ifdef SDL_JOYSTICK_HIDAPI_GIP
+    &SDL_HIDAPI_DriverGIP,
+#endif
 #ifdef SDL_JOYSTICK_HIDAPI_XBOXONE
     &SDL_HIDAPI_DriverXboxOne,
 #endif
+#ifdef SDL_JOYSTICK_HIDAPI_LG4FF
+    &SDL_HIDAPI_DriverLg4ff,
+#endif
+#ifdef SDL_JOYSTICK_HIDAPI_8BITDO
+    &SDL_HIDAPI_Driver8BitDo,
+#endif
+#ifdef SDL_JOYSTICK_HIDAPI_FLYDIGI
+    &SDL_HIDAPI_DriverFlydigi,
+#endif
+#ifdef SDL_JOYSTICK_HIDAPI_SINPUT
+    &SDL_HIDAPI_DriverSInput,
+#endif
+#ifdef SDL_JOYSTICK_HIDAPI_ZUIKI
+    &SDL_HIDAPI_DriverZUIKI,
+#endif
 };
 static int SDL_HIDAPI_numdrivers = 0;
 static SDL_AtomicInt SDL_HIDAPI_updating_devices;
@@ -288,9 +309,13 @@ static SDL_GamepadType SDL_GetJoystickGameControllerProtocol(const char *name, U
             0x1532, // Razer
             0x20d6, // PowerA
             0x24c6, // PowerA
+            0x294b, // Snakebyte
             0x2dc8, // 8BitDo
             0x2e24, // Hyperkin
+            0x2e95, // SCUF
+            0x3285, // Nacon
             0x3537, // GameSir
+            0x366c, // ByoWave
         };
 
         int i;
@@ -338,7 +363,7 @@ static SDL_HIDAPI_DeviceDriver *HIDAPI_GetDeviceDriver(SDL_HIDAPI_Device *device
         return NULL;
     }
 
-    if (device->vendor_id != USB_VENDOR_VALVE) {
+    if (device->vendor_id != USB_VENDOR_VALVE && device->vendor_id != USB_VENDOR_FLYDIGI) {
         if (device->usage_page && device->usage_page != USAGE_PAGE_GENERIC_DESKTOP) {
             return NULL;
         }
@@ -430,7 +455,9 @@ static void HIDAPI_SetupDeviceDriver(SDL_HIDAPI_Device *device, bool *removed) S
     if (device->driver) {
         bool enabled;
 
-        if (device->vendor_id == USB_VENDOR_NINTENDO && device->product_id == USB_PRODUCT_NINTENDO_SWITCH_JOYCON_PAIR) {
+        if (device->vendor_id == USB_VENDOR_NINTENDO &&
+            (device->product_id == USB_PRODUCT_NINTENDO_SWITCH_JOYCON_PAIR ||
+             device->product_id == USB_PRODUCT_NINTENDO_SWITCH2_JOYCON_PAIR)) {
             enabled = SDL_HIDAPI_combine_joycons;
         } else {
             enabled = device->driver->enabled;
@@ -751,11 +778,12 @@ bool HIDAPI_JoystickConnected(SDL_HIDAPI_Device *device, SDL_JoystickID *pJoysti
 
     ++SDL_HIDAPI_numjoysticks;
 
-    SDL_PrivateJoystickAdded(joystickID);
-
     if (pJoystickID) {
         *pJoystickID = joystickID;
     }
+
+    SDL_PrivateJoystickAdded(joystickID);
+
     return true;
 }
 
@@ -1045,12 +1073,21 @@ static bool HIDAPI_CreateCombinedJoyCons(void)
             SDL_zero(info);
             info.path = "nintendo_joycons_combined";
             info.vendor_id = USB_VENDOR_NINTENDO;
-            info.product_id = USB_PRODUCT_NINTENDO_SWITCH_JOYCON_PAIR;
+            if (joycons[0]->product_id == USB_PRODUCT_NINTENDO_SWITCH_JOYCON_LEFT) {
+                info.product_id = USB_PRODUCT_NINTENDO_SWITCH_JOYCON_PAIR;
+            } else {
+                info.product_id = USB_PRODUCT_NINTENDO_SWITCH2_JOYCON_PAIR;
+            }
             info.interface_number = -1;
             info.usage_page = USB_USAGEPAGE_GENERIC_DESKTOP;
             info.usage = USB_USAGE_GENERIC_GAMEPAD;
             info.manufacturer_string = L"Nintendo";
             info.product_string = L"Switch Joy-Con (L/R)";
+            if (children[0]->is_bluetooth || children[1]->is_bluetooth) {
+                info.bus_type = SDL_HID_API_BUS_BLUETOOTH;
+            } else {
+                info.bus_type = SDL_HID_API_BUS_USB;
+            }
 
             combined = HIDAPI_AddDevice(&info, 2, children);
             if (combined && combined->driver) {
