@@ -53,6 +53,7 @@ struct GL_ShaderContext
     PFNGLUNIFORM1IARBPROC glUniform1iARB;
     PFNGLUNIFORM1FARBPROC glUniform1fARB;
     PFNGLUNIFORM3FARBPROC glUniform3fARB;
+    PFNGLUNIFORM4FARBPROC glUniform4fARB;
     PFNGLUSEPROGRAMOBJECTARBPROC glUseProgramObjectARB;
 
     bool GL_ARB_texture_rectangle_supported;
@@ -83,6 +84,73 @@ struct GL_ShaderContext
 "    v_texCoord = vec2(gl_MultiTexCoord0);\n"                   \
 "}"                                                             \
 
+#define RGB_SHADER_PROLOGUE                                     \
+"varying vec4 v_color;\n"                                       \
+"varying vec2 v_texCoord;\n"                                    \
+"uniform sampler2D tex0;\n"                                     \
+"\n"
+
+#define RGB_PIXELART_SHADER_PROLOGUE                            \
+"varying vec4 v_color;\n"                                       \
+"varying vec2 v_texCoord;\n"                                    \
+"uniform sampler2D tex0;\n"                                     \
+"uniform vec4 texel_size; // texel size (xy: texel size, zw: texture dimensions)\n" \
+"\n"
+
+#define PALETTE_SHADER_PROLOGUE                                 \
+"varying vec4 v_color;\n"                                       \
+"varying vec2 v_texCoord;\n"                                    \
+"uniform sampler2D tex0;\n"                                     \
+"uniform sampler2D tex1;\n"                                     \
+"uniform vec4 texel_size; // texel size (xy: texel size, zw: texture dimensions)\n" \
+"\n"
+
+// Implementation with thanks from bgolus:
+// https://discussions.unity.com/t/how-to-make-data-shader-support-bilinear-trilinear/598639/8
+#define PALETTE_SHADER_FUNCTIONS                                    \
+"vec4 SamplePaletteNearest(vec2 uv)\n"                              \
+"{\n"                                                               \
+"    float index = texture2D(tex0, uv).r * 255.0;\n"                \
+"    return texture2D(tex1, vec2((index + 0.5) / 256.0, 0.5));\n"   \
+"}\n"                                                               \
+"\n"                                                                \
+"vec4 SamplePaletteLinear(vec2 uv)\n"                               \
+"{\n"                                                               \
+"    // scale & offset uvs to integer values at texel centers\n"    \
+"    vec2 uv_texels = uv * texel_size.zw + 0.5;\n"                  \
+"\n"                                                                \
+"    // get uvs for the center of the 4 surrounding texels by flooring\n" \
+"    vec4 uv_min_max = vec4((floor(uv_texels) - 0.5) * texel_size.xy, (floor(uv_texels) + 0.5) * texel_size.xy);\n" \
+"\n"                                                                \
+"    // blend factor\n"                                             \
+"    vec2 uv_frac = fract(uv_texels);\n"                            \
+"\n"                                                                \
+"    // sample all 4 texels\n"                                      \
+"    vec4 texelA = SamplePaletteNearest(uv_min_max.xy);\n"          \
+"    vec4 texelB = SamplePaletteNearest(uv_min_max.xw);\n"          \
+"    vec4 texelC = SamplePaletteNearest(uv_min_max.zy);\n"          \
+"    vec4 texelD = SamplePaletteNearest(uv_min_max.zw);\n"          \
+"\n"                                                                \
+"    // bilinear interpolation\n"                                   \
+"    return mix(mix(texelA, texelB, uv_frac.y), mix(texelC, texelD, uv_frac.y), uv_frac.x);\n" \
+"}\n"                                                               \
+"\n"
+
+#define PIXELART_SHADER_FUNCTIONS                                               \
+"vec2 GetPixelArtUV(vec2 uv)\n"                                                 \
+"{\n"                                                                           \
+"    vec2 boxSize = clamp(fwidth(uv) * texel_size.zw, 1e-5, 1.0);\n"            \
+"    vec2 tx = uv * texel_size.zw - 0.5 * boxSize;\n"                           \
+"    vec2 txOffset = smoothstep(vec2(1.0) - boxSize, vec2(1.0), fract(tx));\n"  \
+"    return (floor(tx) + 0.5 + txOffset) * texel_size.xy;\n"                    \
+"}\n"                                                                           \
+"\n"                                                                            \
+"vec4 GetPixelArtSample(vec2 uv)\n"                                             \
+"{\n"                                                                           \
+"    return textureGrad(tex0, GetPixelArtUV(uv), dFdx(v_texCoord), dFdy(v_texCoord));\n"  \
+"}\n"                                                                           \
+"\n"                                                                            \
+
 #define YUV_SHADER_PROLOGUE                                     \
 "varying vec4 v_color;\n"                                       \
 "varying vec2 v_texCoord;\n"                                    \
@@ -236,9 +304,13 @@ struct GL_ShaderContext
  * NOTE: Always use sampler2D, etc here. We'll #define them to the
  *  texture_rectangle versions if we choose to use that extension.
  */
-static const char *shader_source[NUM_SHADERS][2] = {
+static struct {
+    const char *vertex_shader;
+    const char *fragment_shader;
+    const char *fragment_version;
+} shader_source[NUM_SHADERS] = {
     // SHADER_NONE
-    { NULL, NULL },
+    { NULL, NULL, NULL },
 
     // SHADER_SOLID
     {
@@ -250,7 +322,58 @@ static const char *shader_source[NUM_SHADERS][2] = {
 "void main()\n"
 "{\n"
 "    gl_FragColor = v_color;\n"
-"}"
+"}",
+        // fragment version
+        NULL
+    },
+
+    // SHADER_PALETTE_NEAREST
+    {
+        // vertex shader
+        TEXTURE_VERTEX_SHADER,
+        // fragment shader
+        PALETTE_SHADER_PROLOGUE
+        PALETTE_SHADER_FUNCTIONS
+"\n"
+"void main()\n"
+"{\n"
+"    gl_FragColor = SamplePaletteNearest(v_texCoord) * v_color;\n"
+"}",
+        // fragment version
+        NULL
+    },
+
+    // SHADER_PALETTE_LINEAR
+    {
+        // vertex shader
+        TEXTURE_VERTEX_SHADER,
+        // fragment shader
+        PALETTE_SHADER_PROLOGUE
+        PALETTE_SHADER_FUNCTIONS
+"\n"
+"void main()\n"
+"{\n"
+"    gl_FragColor = SamplePaletteLinear(v_texCoord) * v_color;\n"
+"}",
+        // fragment version
+        NULL
+    },
+
+    // SHADER_PALETTE_PIXELART
+    {
+        // vertex shader
+        TEXTURE_VERTEX_SHADER,
+        // fragment shader
+        PALETTE_SHADER_PROLOGUE
+        PALETTE_SHADER_FUNCTIONS
+        PIXELART_SHADER_FUNCTIONS
+"\n"
+"void main()\n"
+"{\n"
+"    gl_FragColor = SamplePaletteLinear(GetPixelArtUV(v_texCoord)) * v_color;\n"
+"}",
+        // fragment version
+        "#version 130\n"
     },
 
     // SHADER_RGB
@@ -258,16 +381,34 @@ static const char *shader_source[NUM_SHADERS][2] = {
         // vertex shader
         TEXTURE_VERTEX_SHADER,
         // fragment shader
-"varying vec4 v_color;\n"
-"varying vec2 v_texCoord;\n"
-"uniform sampler2D tex0;\n"
+        RGB_SHADER_PROLOGUE
 "\n"
 "void main()\n"
 "{\n"
 "    gl_FragColor = texture2D(tex0, v_texCoord);\n"
 "    gl_FragColor.a = 1.0;\n"
 "    gl_FragColor *= v_color;\n"
-"}"
+"}",
+        // fragment version
+        NULL
+    },
+
+    // SHADER_RGB_PIXELART
+    {
+        // vertex shader
+        TEXTURE_VERTEX_SHADER,
+        // fragment shader
+        RGB_PIXELART_SHADER_PROLOGUE
+        PIXELART_SHADER_FUNCTIONS
+"\n"
+"void main()\n"
+"{\n"
+"    gl_FragColor = GetPixelArtSample(v_texCoord);\n"
+"    gl_FragColor.a = 1.0;\n"
+"    gl_FragColor *= v_color;\n"
+"}",
+        // fragment version
+        "#version 130\n"
     },
 
     // SHADER_RGBA
@@ -275,15 +416,33 @@ static const char *shader_source[NUM_SHADERS][2] = {
         // vertex shader
         TEXTURE_VERTEX_SHADER,
         // fragment shader
-"varying vec4 v_color;\n"
-"varying vec2 v_texCoord;\n"
-"uniform sampler2D tex0;\n"
+        RGB_SHADER_PROLOGUE
 "\n"
 "void main()\n"
 "{\n"
 "    gl_FragColor = texture2D(tex0, v_texCoord) * v_color;\n"
-"}"
+"}",
+        // fragment version
+        NULL
     },
+
+    // SHADER_RGBA_PIXELART
+    {
+        // vertex shader
+        TEXTURE_VERTEX_SHADER,
+        // fragment shader
+        RGB_PIXELART_SHADER_PROLOGUE
+        PIXELART_SHADER_FUNCTIONS
+"\n"
+"void main()\n"
+"{\n"
+"    gl_FragColor = GetPixelArtSample(v_texCoord);\n"
+"    gl_FragColor *= v_color;\n"
+"}",
+        // fragment version
+        "#version 130\n"
+    },
+
 #ifdef SDL_HAVE_YUV
     // SHADER_YUV
     {
@@ -291,7 +450,9 @@ static const char *shader_source[NUM_SHADERS][2] = {
         TEXTURE_VERTEX_SHADER,
         // fragment shader
         YUV_SHADER_PROLOGUE
-        YUV_SHADER_BODY
+        YUV_SHADER_BODY,
+        // fragment version
+        NULL
     },
     // SHADER_NV12_RA
     {
@@ -299,7 +460,9 @@ static const char *shader_source[NUM_SHADERS][2] = {
         TEXTURE_VERTEX_SHADER,
         // fragment shader
         NV12_SHADER_PROLOGUE
-        NV12_RA_SHADER_BODY
+        NV12_RA_SHADER_BODY,
+        // fragment version
+        NULL
     },
     // SHADER_NV12_RG
     {
@@ -307,7 +470,9 @@ static const char *shader_source[NUM_SHADERS][2] = {
         TEXTURE_VERTEX_SHADER,
         // fragment shader
         NV12_SHADER_PROLOGUE
-        NV12_RG_SHADER_BODY
+        NV12_RG_SHADER_BODY,
+        // fragment version
+        NULL
     },
     // SHADER_NV21_RA
     {
@@ -315,7 +480,9 @@ static const char *shader_source[NUM_SHADERS][2] = {
         TEXTURE_VERTEX_SHADER,
         // fragment shader
         NV12_SHADER_PROLOGUE
-        NV21_RA_SHADER_BODY
+        NV21_RA_SHADER_BODY,
+        // fragment version
+        NULL
     },
     // SHADER_NV21_RG
     {
@@ -323,20 +490,23 @@ static const char *shader_source[NUM_SHADERS][2] = {
         TEXTURE_VERTEX_SHADER,
         // fragment shader
         NV12_SHADER_PROLOGUE
-        NV21_RG_SHADER_BODY
+        NV21_RG_SHADER_BODY,
+        // fragment version
+        NULL
     },
 #endif // SDL_HAVE_YUV
 };
 
 /* *INDENT-ON* */ // clang-format on
 
-static bool CompileShader(GL_ShaderContext *ctx, GLhandleARB shader, const char *defines, const char *source)
+static bool CompileShader(GL_ShaderContext *ctx, GLhandleARB shader, const char *version, const char *defines, const char *source)
 {
     GLint status;
-    const char *sources[2];
+    const char *sources[3];
 
-    sources[0] = defines;
-    sources[1] = source;
+    sources[0] = version;
+    sources[1] = defines;
+    sources[2] = source;
 
     ctx->glShaderSourceARB(shader, SDL_arraysize(sources), sources, NULL);
     ctx->glCompileShaderARB(shader);
@@ -350,10 +520,15 @@ static bool CompileShader(GL_ShaderContext *ctx, GLhandleARB shader, const char
         info = SDL_small_alloc(char, length + 1, &isstack);
         if (info) {
             ctx->glGetInfoLogARB(shader, length, NULL, info);
-            SDL_LogError(SDL_LOG_CATEGORY_RENDER, "Failed to compile shader:");
-	    SDL_LogError(SDL_LOG_CATEGORY_RENDER, "%s", defines);
-	    SDL_LogError(SDL_LOG_CATEGORY_RENDER, "%s", source);
-	    SDL_LogError(SDL_LOG_CATEGORY_RENDER, "%s", info);
+            SDL_LogDebug(SDL_LOG_CATEGORY_RENDER, "Failed to compile shader:");
+            if (version) {
+                SDL_LogDebug(SDL_LOG_CATEGORY_RENDER, "%s", version);
+            }
+            if (defines) {
+                SDL_LogDebug(SDL_LOG_CATEGORY_RENDER, "%s", defines);
+            }
+            SDL_LogDebug(SDL_LOG_CATEGORY_RENDER, "%s", source);
+            SDL_LogDebug(SDL_LOG_CATEGORY_RENDER, "%s", info);
             SDL_small_free(info, isstack);
         }
         return false;
@@ -367,6 +542,7 @@ static bool CompileShaderProgram(GL_ShaderContext *ctx, int index, GL_ShaderData
     const int num_tmus_bound = 4;
     const char *vert_defines = "";
     const char *frag_defines = "";
+    const char *frag_version = "";
     int i;
     GLint location;
 
@@ -386,19 +562,22 @@ static bool CompileShaderProgram(GL_ShaderContext *ctx, int index, GL_ShaderData
         frag_defines =
             "#define UVCoordScale 1.0\n";
     }
+    if (shader_source[index].fragment_version) {
+        frag_version = shader_source[index].fragment_version;
+    }
 
     // Create one program object to rule them all
     data->program = ctx->glCreateProgramObjectARB();
 
     // Create the vertex shader
     data->vert_shader = ctx->glCreateShaderObjectARB(GL_VERTEX_SHADER_ARB);
-    if (!CompileShader(ctx, data->vert_shader, vert_defines, shader_source[index][0])) {
+    if (!CompileShader(ctx, data->vert_shader, "", vert_defines, shader_source[index].vertex_shader)) {
         return false;
     }
 
     // Create the fragment shader
     data->frag_shader = ctx->glCreateShaderObjectARB(GL_FRAGMENT_SHADER_ARB);
-    if (!CompileShader(ctx, data->frag_shader, frag_defines, shader_source[index][1])) {
+    if (!CompileShader(ctx, data->frag_shader, frag_version, frag_defines, shader_source[index].fragment_shader)) {
         return false;
     }
 
@@ -424,9 +603,18 @@ static bool CompileShaderProgram(GL_ShaderContext *ctx, int index, GL_ShaderData
 
 static void DestroyShaderProgram(GL_ShaderContext *ctx, GL_ShaderData *data)
 {
-    ctx->glDeleteObjectARB(data->vert_shader);
-    ctx->glDeleteObjectARB(data->frag_shader);
-    ctx->glDeleteObjectARB(data->program);
+    if (data->vert_shader) {
+        ctx->glDeleteObjectARB(data->vert_shader);
+        data->vert_shader = 0;
+    }
+    if (data->frag_shader) {
+        ctx->glDeleteObjectARB(data->frag_shader);
+        data->frag_shader = 0;
+    }
+    if (data->program) {
+        ctx->glDeleteObjectARB(data->program);
+        data->program = 0;
+    }
 }
 
 GL_ShaderContext *GL_CreateShaderContext(void)
@@ -466,6 +654,7 @@ GL_ShaderContext *GL_CreateShaderContext(void)
         ctx->glUniform1iARB = (PFNGLUNIFORM1IARBPROC)SDL_GL_GetProcAddress("glUniform1iARB");
         ctx->glUniform1fARB = (PFNGLUNIFORM1FARBPROC)SDL_GL_GetProcAddress("glUniform1fARB");
         ctx->glUniform3fARB = (PFNGLUNIFORM3FARBPROC)SDL_GL_GetProcAddress("glUniform3fARB");
+        ctx->glUniform4fARB = (PFNGLUNIFORM4FARBPROC)SDL_GL_GetProcAddress("glUniform4fARB");
         ctx->glUseProgramObjectARB = (PFNGLUSEPROGRAMOBJECTARBPROC)SDL_GL_GetProcAddress("glUseProgramObjectARB");
         if (ctx->glGetError &&
             ctx->glAttachObjectARB &&
@@ -494,8 +683,7 @@ GL_ShaderContext *GL_CreateShaderContext(void)
     // Compile all the shaders
     for (i = 0; i < NUM_SHADERS; ++i) {
         if (!CompileShaderProgram(ctx, i, &ctx->shaders[i])) {
-            GL_DestroyShaderContext(ctx);
-            return NULL;
+            DestroyShaderProgram(ctx, &ctx->shaders[i]);
         }
     }
 
@@ -503,6 +691,11 @@ GL_ShaderContext *GL_CreateShaderContext(void)
     return ctx;
 }
 
+bool GL_SupportsShader(GL_ShaderContext *ctx, GL_Shader shader)
+{
+    return ctx && ctx->shaders[shader].program;
+}
+
 void GL_SelectShader(GL_ShaderContext *ctx, GL_Shader shader, const float *shader_params)
 {
     GLint location;
@@ -511,23 +704,38 @@ void GL_SelectShader(GL_ShaderContext *ctx, GL_Shader shader, const float *shade
     ctx->glUseProgramObjectARB(program);
 
     if (shader_params && shader_params != ctx->shader_params[shader]) {
-        // YUV shader params are Yoffset, 0, Rcoeff, 0, Gcoeff, 0, Bcoeff, 0
-        location = ctx->glGetUniformLocationARB(program, "Yoffset");
-        if (location >= 0) {
-            ctx->glUniform3fARB(location, shader_params[0], shader_params[1], shader_params[2]);
+        if (shader == SHADER_PALETTE_LINEAR ||
+            shader == SHADER_PALETTE_PIXELART ||
+            shader == SHADER_RGB_PIXELART ||
+            shader == SHADER_RGBA_PIXELART) {
+            location = ctx->glGetUniformLocationARB(program, "texel_size");
+            if (location >= 0) {
+                ctx->glUniform4fARB(location, shader_params[0], shader_params[1], shader_params[2], shader_params[3]);
+            }
         }
-        location = ctx->glGetUniformLocationARB(program, "Rcoeff");
-        if (location >= 0) {
-            ctx->glUniform3fARB(location, shader_params[4], shader_params[5], shader_params[6]);
-        }
-        location = ctx->glGetUniformLocationARB(program, "Gcoeff");
-        if (location >= 0) {
-            ctx->glUniform3fARB(location, shader_params[8], shader_params[9], shader_params[10]);
-        }
-        location = ctx->glGetUniformLocationARB(program, "Bcoeff");
-        if (location >= 0) {
-            ctx->glUniform3fARB(location, shader_params[12], shader_params[13], shader_params[14]);
+
+#ifdef SDL_HAVE_YUV
+        if (shader >= SHADER_YUV) {
+            // YUV shader params are Yoffset, 0, Rcoeff, 0, Gcoeff, 0, Bcoeff, 0
+            location = ctx->glGetUniformLocationARB(program, "Yoffset");
+            if (location >= 0) {
+                ctx->glUniform3fARB(location, shader_params[0], shader_params[1], shader_params[2]);
+            }
+            location = ctx->glGetUniformLocationARB(program, "Rcoeff");
+            if (location >= 0) {
+                ctx->glUniform3fARB(location, shader_params[4], shader_params[5], shader_params[6]);
+            }
+            location = ctx->glGetUniformLocationARB(program, "Gcoeff");
+            if (location >= 0) {
+                ctx->glUniform3fARB(location, shader_params[8], shader_params[9], shader_params[10]);
+            }
+            location = ctx->glGetUniformLocationARB(program, "Bcoeff");
+            if (location >= 0) {
+                ctx->glUniform3fARB(location, shader_params[12], shader_params[13], shader_params[14]);
+            }
         }
+#endif // SDL_HAVE_YUV
+
         ctx->shader_params[shader] = shader_params;
     }
 }
