@@ -45,8 +45,6 @@ SDL_FORCE_INLINE void BlitBto1(SDL_BlitInfo *info, const Uint32 srcbpp)
     dstskip = info->dst_skip;
     map = info->table;
 
-    width += info->leading_skip;
-
     if (srcbpp == 4)
         srcskip += width - (width + 1) / 2;
     else if (srcbpp == 2)
@@ -58,13 +56,7 @@ SDL_FORCE_INLINE void BlitBto1(SDL_BlitInfo *info, const Uint32 srcbpp)
         if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
             while (height--) {
                 Uint8 byte = 0, bit;
-                for (c = 0; c < info->leading_skip; ++c) {
-                    if (!(c & align)) {
-                        byte = *src++;
-                    }
-                    byte >>= srcbpp;
-                }
-                for (; c < width; ++c) {
+                for (c = 0; c < width; ++c) {
                     if (!(c & align)) {
                         byte = *src++;
                     }
@@ -81,13 +73,7 @@ SDL_FORCE_INLINE void BlitBto1(SDL_BlitInfo *info, const Uint32 srcbpp)
         } else {
             while (height--) {
                 Uint8 byte = 0, bit;
-                for (c = 0; c < info->leading_skip; ++c) {
-                    if (!(c & align)) {
-                        byte = *src++;
-                    }
-                    byte <<= srcbpp;
-                }
-                for (; c < width; ++c) {
+                for (c = 0; c < width; ++c) {
                     if (!(c & align)) {
                         byte = *src++;
                     }
@@ -106,13 +92,7 @@ SDL_FORCE_INLINE void BlitBto1(SDL_BlitInfo *info, const Uint32 srcbpp)
         if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
             while (height--) {
                 Uint8 byte = 0, bit;
-                for (c = 0; c < info->leading_skip; ++c) {
-                    if (!(c & align)) {
-                        byte = *src++;
-                    }
-                    byte >>= srcbpp;
-                }
-                for (; c < width; ++c) {
+                for (c = 0; c < width; ++c) {
                     if (!(c & align)) {
                         byte = *src++;
                     }
@@ -129,13 +109,7 @@ SDL_FORCE_INLINE void BlitBto1(SDL_BlitInfo *info, const Uint32 srcbpp)
         } else {
             while (height--) {
                 Uint8 byte = 0, bit;
-                for (c = 0; c < info->leading_skip; ++c) {
-                    if (!(c & align)) {
-                        byte = *src++;
-                    }
-                    byte <<= srcbpp;
-                }
-                for (; c < width; ++c) {
+                for (c = 0; c < width; ++c) {
                     if (!(c & align)) {
                         byte = *src++;
                     }
@@ -173,8 +147,6 @@ SDL_FORCE_INLINE void BlitBto2(SDL_BlitInfo *info, const Uint32 srcbpp)
     dstskip = info->dst_skip / 2;
     map = (Uint16 *)info->table;
 
-    width += info->leading_skip;
-
     if (srcbpp == 4)
         srcskip += width - (width + 1) / 2;
     else if (srcbpp == 2)
@@ -185,13 +157,7 @@ SDL_FORCE_INLINE void BlitBto2(SDL_BlitInfo *info, const Uint32 srcbpp)
     if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte >>= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -208,13 +174,7 @@ SDL_FORCE_INLINE void BlitBto2(SDL_BlitInfo *info, const Uint32 srcbpp)
     } else {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte <<= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -250,8 +210,6 @@ SDL_FORCE_INLINE void BlitBto3(SDL_BlitInfo *info, const Uint32 srcbpp)
     dstskip = info->dst_skip;
     map = info->table;
 
-    width += info->leading_skip;
-
     if (srcbpp == 4)
         srcskip += width - (width + 1) / 2;
     else if (srcbpp == 2)
@@ -262,13 +220,7 @@ SDL_FORCE_INLINE void BlitBto3(SDL_BlitInfo *info, const Uint32 srcbpp)
     if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte >>= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -288,13 +240,7 @@ SDL_FORCE_INLINE void BlitBto3(SDL_BlitInfo *info, const Uint32 srcbpp)
     } else {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte <<= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -334,8 +280,6 @@ SDL_FORCE_INLINE void BlitBto4(SDL_BlitInfo *info, const Uint32 srcbpp)
     dstskip = info->dst_skip / 4;
     map = (Uint32 *)info->table;
 
-    width += info->leading_skip;
-
     if (srcbpp == 4)
         srcskip += width - (width + 1) / 2;
     else if (srcbpp == 2)
@@ -346,13 +290,7 @@ SDL_FORCE_INLINE void BlitBto4(SDL_BlitInfo *info, const Uint32 srcbpp)
     if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte >>= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -369,13 +307,7 @@ SDL_FORCE_INLINE void BlitBto4(SDL_BlitInfo *info, const Uint32 srcbpp)
     } else {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte <<= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -407,8 +339,7 @@ SDL_FORCE_INLINE void BlitBto1Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     Uint8 *palmap = info->table;
     int c;
 
-    width += info->leading_skip;
-
+    // Set up some basic variables
     if (srcbpp == 4)
         srcskip += width - (width + 1) / 2;
     else if (srcbpp == 2)
@@ -420,13 +351,7 @@ SDL_FORCE_INLINE void BlitBto1Key(SDL_BlitInfo *info, const Uint32 srcbpp)
         if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
             while (height--) {
                 Uint8 byte = 0, bit;
-                for (c = 0; c < info->leading_skip; ++c) {
-                    if (!(c & align)) {
-                        byte = *src++;
-                    }
-                    byte >>= srcbpp;
-                }
-                for (; c < width; ++c) {
+                for (c = 0; c < width; ++c) {
                     if (!(c & align)) {
                         byte = *src++;
                     }
@@ -443,13 +368,7 @@ SDL_FORCE_INLINE void BlitBto1Key(SDL_BlitInfo *info, const Uint32 srcbpp)
         } else {
             while (height--) {
                 Uint8 byte = 0, bit;
-                for (c = 0; c < info->leading_skip; ++c) {
-                    if (!(c & align)) {
-                        byte = *src++;
-                    }
-                    byte <<= srcbpp;
-                }
-                for (; c < width; ++c) {
+                for (c = 0; c < width; ++c) {
                     if (!(c & align)) {
                         byte = *src++;
                     }
@@ -468,13 +387,7 @@ SDL_FORCE_INLINE void BlitBto1Key(SDL_BlitInfo *info, const Uint32 srcbpp)
         if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
             while (height--) {
                 Uint8 byte = 0, bit;
-                for (c = 0; c < info->leading_skip; ++c) {
-                    if (!(c & align)) {
-                        byte = *src++;
-                    }
-                    byte >>= srcbpp;
-                }
-                for (; c < width; ++c) {
+                for (c = 0; c < width; ++c) {
                     if (!(c & align)) {
                         byte = *src++;
                     }
@@ -491,13 +404,7 @@ SDL_FORCE_INLINE void BlitBto1Key(SDL_BlitInfo *info, const Uint32 srcbpp)
         } else {
             while (height--) {
                 Uint8 byte = 0, bit;
-                for (c = 0; c < info->leading_skip; ++c) {
-                    if (!(c & align)) {
-                        byte = *src++;
-                    }
-                    byte <<= srcbpp;
-                }
-                for (; c < width; ++c) {
+                for (c = 0; c < width; ++c) {
                     if (!(c & align)) {
                         byte = *src++;
                     }
@@ -530,8 +437,7 @@ SDL_FORCE_INLINE void BlitBto2Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     Uint8 *palmap = info->table;
     int c;
 
-    width += info->leading_skip;
-
+    // Set up some basic variables
     if (srcbpp == 4)
         srcskip += width - (width + 1) / 2;
     else if (srcbpp == 2)
@@ -543,13 +449,7 @@ SDL_FORCE_INLINE void BlitBto2Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte >>= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -566,13 +466,7 @@ SDL_FORCE_INLINE void BlitBto2Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     } else {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte <<= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -604,8 +498,7 @@ SDL_FORCE_INLINE void BlitBto3Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     Uint8 *palmap = info->table;
     int c;
 
-    width += info->leading_skip;
-
+    // Set up some basic variables
     if (srcbpp == 4)
         srcskip += width - (width + 1) / 2;
     else if (srcbpp == 2)
@@ -616,13 +509,7 @@ SDL_FORCE_INLINE void BlitBto3Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte >>= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -639,13 +526,7 @@ SDL_FORCE_INLINE void BlitBto3Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     } else {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte <<= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -677,8 +558,7 @@ SDL_FORCE_INLINE void BlitBto4Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     Uint8 *palmap = info->table;
     int c;
 
-    width += info->leading_skip;
-
+    // Set up some basic variables
     if (srcbpp == 4)
         srcskip += width - (width + 1) / 2;
     else if (srcbpp == 2)
@@ -690,13 +570,7 @@ SDL_FORCE_INLINE void BlitBto4Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte >>= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -713,13 +587,7 @@ SDL_FORCE_INLINE void BlitBto4Key(SDL_BlitInfo *info, const Uint32 srcbpp)
     } else {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte <<= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -749,13 +617,12 @@ static void BlitBtoNAlpha(SDL_BlitInfo *info)
     const SDL_PixelFormatDetails *dstfmt = info->dst_fmt;
     int srcbpp, dstbpp;
     int c;
-    Uint32 pixel, mask, align;
+    Uint32 pixelvalue, mask, align;
     unsigned sR, sG, sB;
     unsigned dR, dG, dB, dA;
     const unsigned A = info->a;
 
-    width += info->leading_skip;
-
+    // Set up some basic variables
     srcbpp = srcfmt->bytes_per_pixel;
     dstbpp = dstfmt->bytes_per_pixel;
     if (srcbpp == 4)
@@ -770,13 +637,7 @@ static void BlitBtoNAlpha(SDL_BlitInfo *info)
     if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte >>= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -785,7 +646,7 @@ static void BlitBtoNAlpha(SDL_BlitInfo *info)
                     sR = srcpal[bit].r;
                     sG = srcpal[bit].g;
                     sB = srcpal[bit].b;
-                    DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixel, dR, dG, dB, dA);
+                    DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixelvalue, dR, dG, dB, dA);
                     ALPHA_BLEND_RGBA(sR, sG, sB, A, dR, dG, dB, dA);
                     ASSEMBLE_RGBA(dst, dstbpp, dstfmt, dR, dG, dB, dA);
                 }
@@ -798,13 +659,7 @@ static void BlitBtoNAlpha(SDL_BlitInfo *info)
     } else {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte <<= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -813,7 +668,7 @@ static void BlitBtoNAlpha(SDL_BlitInfo *info)
                     sR = srcpal[bit].r;
                     sG = srcpal[bit].g;
                     sB = srcpal[bit].b;
-                    DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixel, dR, dG, dB, dA);
+                    DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixelvalue, dR, dG, dB, dA);
                     ALPHA_BLEND_RGBA(sR, sG, sB, A, dR, dG, dB, dA);
                     ASSEMBLE_RGBA(dst, dstbpp, dstfmt, dR, dG, dB, dA);
                 }
@@ -839,14 +694,13 @@ static void BlitBtoNAlphaKey(SDL_BlitInfo *info)
     const SDL_Color *srcpal = info->src_pal->colors;
     int srcbpp, dstbpp;
     int c;
-    Uint32 pixel, mask, align;
+    Uint32 pixelvalue, mask, align;
     unsigned sR, sG, sB;
     unsigned dR, dG, dB, dA;
     const unsigned A = info->a;
     Uint32 ckey = info->colorkey;
 
-    width += info->leading_skip;
-
+    // Set up some basic variables
     srcbpp = srcfmt->bytes_per_pixel;
     dstbpp = dstfmt->bytes_per_pixel;
     if (srcbpp == 4)
@@ -861,13 +715,7 @@ static void BlitBtoNAlphaKey(SDL_BlitInfo *info)
     if (SDL_PIXELORDER(info->src_fmt->format) == SDL_BITMAPORDER_4321) {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte >>= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -876,7 +724,7 @@ static void BlitBtoNAlphaKey(SDL_BlitInfo *info)
                     sR = srcpal[bit].r;
                     sG = srcpal[bit].g;
                     sB = srcpal[bit].b;
-                    DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixel, dR, dG, dB, dA);
+                    DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixelvalue, dR, dG, dB, dA);
                     ALPHA_BLEND_RGBA(sR, sG, sB, A, dR, dG, dB, dA);
                     ASSEMBLE_RGBA(dst, dstbpp, dstfmt, dR, dG, dB, dA);
                 }
@@ -889,13 +737,7 @@ static void BlitBtoNAlphaKey(SDL_BlitInfo *info)
     } else {
         while (height--) {
             Uint8 byte = 0, bit;
-            for (c = 0; c < info->leading_skip; ++c) {
-                if (!(c & align)) {
-                    byte = *src++;
-                }
-                byte <<= srcbpp;
-            }
-            for (; c < width; ++c) {
+            for (c = 0; c < width; ++c) {
                 if (!(c & align)) {
                     byte = *src++;
                 }
@@ -904,7 +746,7 @@ static void BlitBtoNAlphaKey(SDL_BlitInfo *info)
                     sR = srcpal[bit].r;
                     sG = srcpal[bit].g;
                     sB = srcpal[bit].b;
-                    DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixel, dR, dG, dB, dA);
+                    DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixelvalue, dR, dG, dB, dA);
                     ALPHA_BLEND_RGBA(sR, sG, sB, A, dR, dG, dB, dA);
                     ASSEMBLE_RGBA(dst, dstbpp, dstfmt, dR, dG, dB, dA);
                 }
